From e051e51a244fc9f903b4211bdb7e68d366a7a538 Mon Sep 17 00:00:00 2001
From: David Adam <david.adam.cnrs@gmail.com>
Date: Wed, 18 Jun 2008 02:21:35 +0200
Subject: [PATCH] Implement the basic functions for MatrixStack

---
 dlls/d3dx8/d3dx8_private.h |    8 ++++-
 dlls/d3dx8/math.c          |   55 ++++++++++++++++++++++++++++++++++++++-----
 2 files changed, 54 insertions(+), 9 deletions(-)

diff --git a/dlls/d3dx8/d3dx8_private.h b/dlls/d3dx8/d3dx8_private.h
index 923f285..fcad36f 100644
--- a/dlls/d3dx8/d3dx8_private.h
+++ b/dlls/d3dx8/d3dx8_private.h
@@ -79,6 +79,9 @@ struct ID3DXFontImpl
 /*****************************************************************************
  * ID3DXMatrixStackImpl implementation structure
  */
+
+#define INIT_STACK_SIZE 32
+
 struct ID3DXMatrixStackImpl
 {
   /* IUnknown fields */
@@ -86,8 +89,9 @@ struct ID3DXMatrixStackImpl
   LONG                   ref;
 
   /* ID3DXMatrixStack fields */
-  int current;
-  D3DXMATRIX *stack;
+  unsigned int current;
+  unsigned int stack_size; /*counter for the size of the array that represents the stack */
+  D3DXMATRIX *matrix;
 };
 
 #endif /*__WINE_D3DX8_PRIVATE_H */
diff --git a/dlls/d3dx8/math.c b/dlls/d3dx8/math.c
index a65a934..4850964 100644
--- a/dlls/d3dx8/math.c
+++ b/dlls/d3dx8/math.c
@@ -584,15 +584,22 @@ HRESULT WINAPI D3DXCreateMatrixStack(DWORD flags, LPD3DXMATRIXSTACK* ppstack)
 {
     ID3DXMatrixStackImpl* object;
 
+
+    TRACE("flags = %d, ppstack = %p\n",flags, *ppstack);
+    *ppstack = NULL;
     object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(ID3DXMatrixStackImpl));
-    if ( object == NULL )
-    {
-     *ppstack = NULL;
-     return E_OUTOFMEMORY;
-    }
+    if ( object == NULL ) return E_OUTOFMEMORY;
     object->lpVtbl = &ID3DXMatrixStack_Vtbl;
     object->ref = 1;
     object->current = 0;
+    object->matrix = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, INIT_STACK_SIZE * sizeof(D3DXMATRIX));
+    if ( object->matrix == NULL )
+    {
+     HeapFree(GetProcessHeap(), 0, object);
+     return E_OUTOFMEMORY;
+    }
+    D3DXMatrixIdentity(&(object->matrix[0]));
+    object->stack_size = INIT_STACK_SIZE;
     *ppstack = (LPD3DXMATRIXSTACK)object;
     return D3D_OK;
 }
@@ -666,14 +673,48 @@ static HRESULT WINAPI ID3DXMatrixStackImpl_MultMatrixLocal(ID3DXMatrixStack *ifa
 static HRESULT WINAPI ID3DXMatrixStackImpl_Pop(ID3DXMatrixStack *iface)
 {
     ID3DXMatrixStackImpl *This = (ID3DXMatrixStackImpl *)iface;
-    FIXME("(%p) : stub\n",This);
+
+    TRACE ("This = %p, current = %d\n", This, This->current);
+    if ( !This->current ) return D3D_OK; /*It is authorized to pop the first matrix (that is when current == 0). */
+    if ( (This->current <= This->stack_size / 4) && (This->stack_size >= INIT_STACK_SIZE * 2) )
+    {
+     unsigned int size;
+     D3DXMATRIX *temp = NULL;
+
+     size = This->stack_size / 2;
+     temp = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, This->matrix, size * sizeof(D3DXMATRIX));
+     if ( temp )
+     {
+      This->stack_size = size;
+      This->matrix = temp;
+     }
+    }
+    This->current = This->current - 1;
     return D3D_OK;
 }
 
 static HRESULT WINAPI ID3DXMatrixStackImpl_Push(ID3DXMatrixStack *iface)
 {
     ID3DXMatrixStackImpl *This = (ID3DXMatrixStackImpl *)iface;
-    FIXME("(%p) : stub\n",This);
+
+    TRACE ("This = %p, current = %d\n", This, This->current);
+    This->current = This->current + 1;
+    if ( This->current == This->stack_size )
+    {
+     unsigned int size;
+     D3DXMATRIX *temp = NULL;
+
+     size = This->stack_size * 2;
+     temp = HeapReAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, This->matrix, size * sizeof(D3DXMATRIX));
+     if ( !temp )
+     {
+      This->current = This->current - 1;
+      return E_OUTOFMEMORY;
+     }
+     This->stack_size = size;
+     This->matrix = temp;
+    }
+    This->matrix[This->current] = This->matrix[This->current-1];
     return D3D_OK;
 }
 
-- 
1.5.4.3

